@@ -5,11 +5,12 @@
 # definitions of spatial distribution characters
 
 import math
+import warnings
 
 import networkx as nx
 import numpy as np
 import pandas as pd
-from tqdm import tqdm  # progress bar
+from tqdm.auto import tqdm  # progress bar
 
 from .utils import _azimuth
 
@@ -323,6 +324,7 @@ class StreetAlignment:
         )
 
         self.series = np.abs(merged[orientations] - merged["_orientation"])
+        self.series.index = left.index
 
 
 class CellAlignment:
