@@ -3,18 +3,18 @@
 
 # elements.py
 # generating derived elements (street edge, block)
+from distutils.version import LooseVersion
+
 import geopandas as gpd
 import libpysal
+import warnings
 import numpy as np
 import pygeos
 import pandas as pd
 from scipy.spatial import Voronoi
 from shapely.geometry.base import BaseGeometry
 from shapely.ops import polygonize
-from tqdm import tqdm
-
-# TODO: this should not be needed with shapely 2.0
-from geopandas._vectorized import _pygeos_to_shapely
+from tqdm.auto import tqdm
 
 __all__ = [
     "buffered_limit",
@@ -26,6 +26,8 @@ __all__ = [
     "get_network_ratio",
 ]
 
+GPD_10 = str(gpd.__version__) >= LooseVersion("0.10")
+
 
 def buffered_limit(gdf, buffer=100):
     """
@@ -200,6 +202,13 @@ class Tessellation:
         self.segment = segment
         self.enclosure_id = enclosure_id
 
+        if gdf.crs and gdf.crs.is_geographic:
+            raise ValueError(
+                "Geometry is in a geographic CRS. "
+                "Use 'GeoDataFrame.to_crs()' to re-project geometries to a "
+                "projected CRS before using Tessellation.\n",
+            )
+
         if limit is not None and enclosures is not None:
             raise ValueError(
                 "Both `limit` and `enclosures` cannot be passed together. "
@@ -268,8 +277,10 @@ class Tessellation:
             objects.loc[mask, objects.geometry.name] = objects[mask].buffer(
                 -shrink, cap_style=2, join_style=2
             )
-
-        objects = objects.reset_index(drop=True).explode()
+        if GPD_10:
+            objects = objects.reset_index(drop=True).explode(ignore_index=True)
+        else:
+            objects = objects.reset_index(drop=True).explode(ignore_index=True)
         objects = objects.set_index(unique_id)
 
         print("Generating input point array...") if verbose else None
@@ -367,7 +378,6 @@ class Tessellation:
         ids_original = list(orig_gdf[unique_id])
         ids_generated = list(tesselation[unique_id])
         if len(ids_original) != len(ids_generated):
-            import warnings
 
             self.collapsed = set(ids_original).difference(ids_generated)
             warnings.warn(
@@ -381,8 +391,6 @@ class Tessellation:
             unique_id
         ]
         if len(self.multipolygons) > 0:
-            import warnings
-
             warnings.warn(
                 "Tessellation contains MultiPolygon elements. Initial objects should "
                 f"be edited. unique_id of affected elements: {list(self.multipolygons)}"
@@ -442,6 +450,7 @@ class Tessellation:
 
         """
         enclosures = enclosures.reset_index(drop=True)
+        enclosures["position"] = range(len(enclosures))
 
         # determine which polygons should be split
         inp, res = buildings.sindex.query_bulk(
@@ -453,13 +462,11 @@ class Tessellation:
 
         if use_dask:
             try:
-                import dask.dataframe as dd
+                import dask.bag as db
                 from dask.system import cpu_count
             except ImportError:
                 use_dask = False
 
-                import warnings
-
                 warnings.warn(
                     "dask.dataframe could not be imported. Setting `use_dask=False`."
                 )
@@ -467,18 +474,18 @@ class Tessellation:
         if use_dask:
             if n_chunks is None:
                 n_chunks = cpu_count() - 1 if cpu_count() > 1 else 1
-            # initialize dask.series
-            ds = dd.from_array(splits, chunksize=len(splits) // n_chunks)
+            # initialize dask.bag
+            bag = db.from_sequence(splits, npartitions=n_chunks)
             # generate enclosed tessellation using dask
-            new = (
-                ds.apply(
-                    self._tess,
-                    meta=(None, "object"),
-                    args=(enclosures, buildings, inp, res, threshold, unique_id),
-                )
-                .compute()
-                .to_list()
-            )
+            new = bag.map(
+                self._tess,
+                enclosures,
+                buildings,
+                inp,
+                res,
+                threshold,
+                unique_id,
+            ).compute()
 
         else:
             new = [
@@ -497,12 +504,14 @@ class Tessellation:
 
         # finalise the result
         clean_blocks = enclosures.drop(splits)
-        clean_blocks.loc[single, "uID"] = clean_blocks.loc[single][enclosure_id].apply(
+        clean_blocks.loc[single, "uID"] = clean_blocks.loc[single, "position"].apply(
             lambda ix: buildings.iloc[res[inp == ix][0]][unique_id]
         )
         tessellation = pd.concat(new)
 
-        return tessellation.append(clean_blocks).reset_index(drop=True)
+        return tessellation.append(clean_blocks.drop(columns="position")).reset_index(
+            drop=True
+        )
 
     def _tess(
         self,
@@ -520,7 +529,7 @@ class Tessellation:
         within = blg[
             pygeos.area(pygeos.intersection(blg.geometry.values.data, poly))
             > (pygeos.area(blg.geometry.values.data) * threshold)
-        ]
+        ].copy()
         if len(within) > 1:
             tess = self._morphological_tessellation(
                 within,
@@ -623,40 +632,67 @@ class Blocks:
             tessellation,
             gpd.GeoDataFrame(geometry=edges.buffer(0.001)),
             how="difference",
-        ).explode()
+        )
+        if GPD_10:
+            cut = cut.explode(ignore_index=True)
+        else:
+            cut = cut.explode()
 
         W = libpysal.weights.Queen.from_dataframe(cut, silence_warnings=True)
         cut["component"] = W.component_labels
         buildings_c = buildings.copy()
         buildings_c.geometry = buildings_c.representative_point()  # make points
-        centroids_tempID = gpd.sjoin(
-            buildings_c,
-            cut[[cut.geometry.name, "component"]],
-            how="left",
-            op="intersects",
-        )
+        if GPD_10:
+            centroids_tempID = gpd.sjoin(
+                buildings_c,
+                cut[[cut.geometry.name, "component"]],
+                how="left",
+                predicate="intersects",
+            )
+        else:
+            centroids_tempID = gpd.sjoin(
+                buildings_c,
+                cut[[cut.geometry.name, "component"]],
+                how="left",
+                op="intersects",
+            )
+
         cells_copy = tessellation[[unique_id, tessellation.geometry.name]].merge(
             centroids_tempID[[unique_id, "component"]], on=unique_id, how="left"
         )
-        blocks = cells_copy.dissolve(by="component").explode().reset_index(drop=True)
+        if GPD_10:
+            blocks = cells_copy.dissolve(by="component").explode(ignore_index=True)
+        else:
+            blocks = (
+                cells_copy.dissolve(by="component").explode().reset_index(drop=True)
+            )
         blocks[id_name] = range(len(blocks))
         blocks[blocks.geometry.name] = gpd.GeoSeries(
             pygeos.polygons(blocks.exterior.values.data), crs=blocks.crs
         )
         blocks = blocks[[id_name, blocks.geometry.name]]
 
-        centroids_w_bl_ID2 = gpd.sjoin(buildings_c, blocks, how="left", op="intersects")
+        if GPD_10:
+            centroids_w_bl_ID2 = gpd.sjoin(
+                buildings_c, blocks, how="left", predicate="intersects"
+            )
+        else:
+            centroids_w_bl_ID2 = gpd.sjoin(
+                buildings_c, blocks, how="left", op="intersects"
+            )
         bl_ID_to_uID = centroids_w_bl_ID2[[unique_id, id_name]]
 
         buildings_m = buildings[[unique_id]].merge(
             bl_ID_to_uID, on=unique_id, how="left"
         )
         self.buildings_id = buildings_m[id_name]
+        self.buildings_id.index = self.buildings.index
 
         cells_m = tessellation[[unique_id]].merge(
             bl_ID_to_uID, on=unique_id, how="left"
         )
         self.tessellation_id = cells_m[id_name]
+        self.tessellation_id.index = self.tessellation.index
 
         self.blocks = blocks
 
@@ -713,12 +749,10 @@ def get_network_id(left, right, network_id, min_size=100, verbose=True):
         right["mm_nid"] = network_id
         network_id = "mm_nid"
 
-    print("Generating centroids...") if verbose else None
     buildings_c = left.copy()
 
     buildings_c[buildings_c.geometry.name] = buildings_c.centroid  # make centroids
 
-    print("Generating rtree...") if verbose else None
     idx = right.sindex
 
     # TODO: use sjoin nearest once done
@@ -743,11 +777,9 @@ def get_network_id(left, right, network_id, min_size=100, verbose=True):
         else:
             result.append(nid)
 
-    series = pd.Series(result)
+    series = pd.Series(result, index=left.index)
 
     if series.isnull().any():
-        import warnings
-
         warnings.warn(
             "Some objects were not attached to the network. "
             "Set larger min_size. {} affected elements".format(sum(series.isnull()))
@@ -928,6 +960,7 @@ def get_network_ratio(df, edges, initial_buffer=500):
     nans = df.index.difference(edge_dicts.index)
     buffered = df.iloc[nans].buffer(initial_buffer)
     additional = []
+
     for orig, geom in zip(df.iloc[nans].geometry, buffered.geometry):
         query = edges.sindex.query(geom, predicate="intersects")
         b = initial_buffer
@@ -946,7 +979,11 @@ def get_network_ratio(df, edges, initial_buffer=500):
 
 
 def enclosures(
-    primary_barriers, limit=None, additional_barriers=None, enclosure_id="eID"
+    primary_barriers,
+    limit=None,
+    additional_barriers=None,
+    enclosure_id="eID",
+    clip=False,
 ):
     """
     Generate enclosures based on passed barriers.
@@ -962,8 +999,9 @@ def enclosures(
     primary_barriers : GeoDataFrame, GeoSeries
         GeoDataFrame or GeoSeries containing primary barriers.
         (Multi)LineString geometry is expected.
-    limit : GeoDataFrame, GeoSeries (default None)
-        GeoDataFrame or GeoSeries containing external limit of enclosures,
+    limit : GeoDataFrame, GeoSeries, shapely geometry (default None)
+        GeoDataFrame, GeoSeries or shapely geometry containing external limit
+        of enclosures,
         i.e. the area which gets partitioned. If None is passed,
         the internal area of ``primary_barriers`` will be used.
     additional_barriers : GeoDataFrame
@@ -971,6 +1009,9 @@ def enclosures(
         (Multi)LineString geometry is expected.
     enclosure_id : str (default 'eID')
         name of the enclosure_id (to be created).
+    clip : bool (default False)
+        if True, returns enclosures with representative point within the limit
+        (if given). Requires ``limit`` composed of Polygon or MultiPolygon geometries.
 
     Returns
     -------
@@ -983,9 +1024,13 @@ def enclosures(
 
     """
     if limit is not None:
+        if isinstance(limit, BaseGeometry):
+            limit = gpd.GeoSeries([limit])
         if limit.geom_type.isin(["Polygon", "MultiPolygon"]).any():
-            limit = limit.boundary
-        barriers = pd.concat([primary_barriers.geometry, limit.geometry])
+            limit_b = limit.boundary
+        else:
+            limit_b = limit
+        barriers = pd.concat([primary_barriers.geometry, limit_b.geometry])
     else:
         barriers = primary_barriers
     unioned = barriers.unary_union
@@ -1017,11 +1062,9 @@ def enclosures(
             union = pygeos.union_all(
                 np.append(crossing_ins, pygeos.boundary(poly))
             )  # union
-            polygons = np.array(
-                list(polygonize(_pygeos_to_shapely(union)))
-            )  # polygonize
+            polygons = pygeos.get_parts(pygeos.polygonize([union]))  # polygonize
             within = pygeos.covered_by(
-                pygeos.from_shapely(polygons), buf
+                polygons, buf
             )  # keep only those within original polygon
             new += list(polygons[within])
 
@@ -1032,8 +1075,25 @@ def enclosures(
             .reset_index(drop=True)
         ).set_crs(primary_barriers.crs)
 
-        return gpd.GeoDataFrame(
+        final_enclosures = gpd.GeoDataFrame(
             {enclosure_id: range(len(final_enclosures))}, geometry=final_enclosures
         )
 
-    return gpd.GeoDataFrame({enclosure_id: range(len(enclosures))}, geometry=enclosures)
+    else:
+        final_enclosures = gpd.GeoDataFrame(
+            {enclosure_id: range(len(enclosures))}, geometry=enclosures
+        )
+
+    if clip and limit is not None:
+        if not limit.geom_type.isin(["Polygon", "MultiPolygon"]).all():
+            raise TypeError(
+                "`limit` requires a GeoDataFrame or GeoSeries with Polygon or "
+                "MultiPolygon geometry to be used with clip=True."
+            )
+        _, encl_index = final_enclosures.representative_point().sindex.query_bulk(
+            limit.geometry, predicate="contains"
+        )
+        keep = np.unique(encl_index)
+        return final_enclosures.iloc[keep]
+
+    return final_enclosures
